package com.tzld.piaoquan.api.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.api.dao.mapper.GuaranteesVideoMapper;
import com.tzld.piaoquan.api.dao.mapper.MessageAttachmentMapper;
import com.tzld.piaoquan.api.service.MessageAttachmentService;
import com.tzld.piaoquan.common.common.base.CommonResponse;
import com.tzld.piaoquan.common.common.enums.MessageAttachmentTypeEnum;
import com.tzld.piaoquan.common.component.HttpPoolClient;
import com.tzld.piaoquan.common.model.bo.VideoDetail;
import com.tzld.piaoquan.common.model.bo.VideoParam;
import com.tzld.piaoquan.common.model.po.GuaranteesVideo;
import com.tzld.piaoquan.common.model.po.GuaranteesVideoExample;
import com.tzld.piaoquan.common.model.po.MessageAttachment;
import com.tzld.piaoquan.common.model.po.MessageAttachmentExample;
import com.tzld.piaoquan.common.model.vo.GuaranteedParam;
import com.tzld.piaoquan.common.utils.DateUtil;
import com.tzld.piaoquan.common.utils.LarkRobotUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.*;

import static com.tzld.piaoquan.common.common.constant.MessageConstant.MAX_VIDEO_NUM;
import static com.tzld.piaoquan.common.common.constant.MessageConstant.appid;
import static com.tzld.piaoquan.common.common.constant.OtherServerURL.POST_VIDEO_DETAIL_URL;
import static com.tzld.piaoquan.common.common.constant.TimeConstant.MILLISECOND_DAY;


@Slf4j
@Service
public class MessageAttachmentServiceImpl implements MessageAttachmentService {

    @Autowired
    private HttpPoolClient httpPoolClient;

    @Autowired
    private MessageAttachmentMapper messageAttachmentMapper;

    @Autowired
    private GuaranteesVideoMapper guaranteesVideoMapper;

    @Override
    public void addMiniProgram(List<MessageAttachment> messageAttachmentList, Long createTime) {
        for (MessageAttachment messageAttachment : messageAttachmentList) {
            MessageAttachmentExample example = new MessageAttachmentExample();
            example.createCriteria()
                    .andAppidEqualTo(messageAttachment.getAppid())
                    .andTypeEqualTo(messageAttachment.getType())
                    .andMiniprogramVideoIdEqualTo(messageAttachment.getMiniprogramVideoId())
                    .andStaffIdEqualTo(messageAttachment.getStaffId());
            List<MessageAttachment> messageAttachments = messageAttachmentMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(messageAttachments)) {
                if (createTime != null) {
                    messageAttachment.setSendTime(new Date(createTime));
                }
                messageAttachmentMapper.insert(messageAttachment);
                continue;
            }
            messageAttachment.setId(messageAttachments.get(0).getId());
            messageAttachmentMapper.updateByPrimaryKeySelective(messageAttachment);
        }
    }


    public CommonResponse<Void> createGuaranteedMiniProgram(GuaranteedParam guaranteedParam) {
        if (isInvalidGuaranteedParam(guaranteedParam)) {
            return CommonResponse.create(500, "参数错误");
        }

        Set<Long> videoIds = new HashSet<>();
        for (VideoParam videoParam : guaranteedParam.getVideoParamList()) {
            processVideoParam(videoParam, guaranteedParam.getDate(), videoIds);
        }

        Map<Long, VideoDetail> coverMap = getVideoDetail(videoIds);
        Map<Long, MessageAttachment> messageAttachmentMap = createMessageAttachmentMap(videoIds, coverMap);

        List<MessageAttachment> messageAttachmentList = new ArrayList<>();
        for (VideoParam videoParam : guaranteedParam.getVideoParamList()) {
            addMessageAttachments(videoParam, messageAttachmentMap, guaranteedParam.getDate(), messageAttachmentList);
        }

        addMiniProgram(messageAttachmentList, null);
        return CommonResponse.success();
    }

    private boolean isInvalidGuaranteedParam(GuaranteedParam guaranteedParam) {
        return guaranteedParam == null ||
                StringUtils.isEmpty(guaranteedParam.getDate()) ||
                CollectionUtils.isEmpty(guaranteedParam.getVideoParamList());
    }


    private void processVideoParam(VideoParam videoParam, String date, Set<Long> videoIds) {
        if (CollectionUtils.isEmpty(videoParam.getVideoIds()) || videoParam.getVideoIds().size() < MAX_VIDEO_NUM) {
            LarkRobotUtil.sendMessage("保底视频数量异常，请查看" + date);
        }
        List<Long> staffIds = Optional.ofNullable(videoParam.getStaffIds()).orElseGet(() -> {
            List<Long> newStaffIds = new ArrayList<>();
            if (videoParam.getStaffId() != null) {
                newStaffIds.add(videoParam.getStaffId());
            }
            return newStaffIds;
        });
        for (Long staffId : staffIds) {
            checkVideoSendTime(videoParam, staffId);
            videoIds.addAll(videoParam.getVideoIds());
        }
    }

    private void checkVideoSendTime(VideoParam videoParam, Long staffId) {
        for (Long videoId : videoParam.getVideoIds()) {
            MessageAttachmentExample example = new MessageAttachmentExample();
            example.createCriteria().andMiniprogramVideoIdEqualTo(videoId).andStaffIdEqualTo(staffId);
            List<MessageAttachment> messageAttachmentList = messageAttachmentMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(messageAttachmentList)) {
                continue;
            }
            MessageAttachment messageAttachment = messageAttachmentList.get(0);
            if (messageAttachment.getSendTime() != null &&
                    DateUtil.dateDifference(new Date(), messageAttachment.getSendTime()) < 90 * MILLISECOND_DAY) {
                LarkRobotUtil.sendMessage("保底视频90天内已发送，请查看videoId=" + videoId);
                throw new RuntimeException("保底视频90天内已发送，请查看videoId=" + videoId);
            }
        }
    }

    private Map<Long, MessageAttachment> createMessageAttachmentMap(Set<Long> videoIds, Map<Long, VideoDetail> coverMap) {
        Map<Long, MessageAttachment> messageAttachmentMap = new HashMap<>();
        for (Long videoId : videoIds) {
            VideoDetail videoDetail = coverMap.get(videoId);
            if (videoDetail == null || StringUtils.isEmpty(videoDetail.getCover()) || StringUtils.isEmpty(videoDetail.getTitle())) {
                LarkRobotUtil.sendMessage("获取视频详情异常，请查看" + videoId);
                throw new RuntimeException("获取视频详情异常");
            }
            MessageAttachment messageAttachment = new MessageAttachment();
            messageAttachment.setMiniprogramVideoId(videoId);
            messageAttachment.setType(MessageAttachmentTypeEnum.MINI_PROGRAM.getType());
            messageAttachment.setCover(videoDetail.getCover());
            messageAttachment.setTitle(videoDetail.getTitle());
            messageAttachment.setAppid(appid);
            messageAttachmentMap.put(videoId, messageAttachment);
        }
        return messageAttachmentMap;
    }

    private void addMessageAttachments(VideoParam videoParam, Map<Long, MessageAttachment> messageAttachmentMap, String date, List<MessageAttachment> messageAttachmentList) {
        List<Long> staffIds = Optional.ofNullable(videoParam.getStaffIds()).orElseGet(() -> {
            List<Long> newStaffIds = new ArrayList<>();
            if (videoParam.getStaffId() != null) {
                newStaffIds.add(videoParam.getStaffId());
            }
            return newStaffIds;
        });

        for (Long staffId : staffIds) {
            if (staffId == 0) {
                addGuaranteesVideo(date, staffId, videoParam.getVideoIds());
                continue;
            }

            for (Long videoId : videoParam.getVideoIds()) {
                MessageAttachment messageAttachment = messageAttachmentMap.get(videoId);
                if (messageAttachment == null) {
                    continue;
                }

                MessageAttachment newMessageAttachment = new MessageAttachment();
                BeanUtils.copyProperties(messageAttachment, newMessageAttachment);
                newMessageAttachment.setStaffId(staffId);
                messageAttachmentList.add(newMessageAttachment);
            }
            addGuaranteesVideo(date, staffId, videoParam.getVideoIds());
        }
    }

    private void addGuaranteesVideo(String date, Long staffId, List<Long> videoIds) {
        GuaranteesVideoExample example = new GuaranteesVideoExample();
        example.createCriteria().andIsDeleteEqualTo(0).andDateEqualTo(date).andStaffIdEqualTo(staffId);
        List<GuaranteesVideo> guaranteesVideos = guaranteesVideoMapper.selectByExample(example);
        for (GuaranteesVideo guaranteesVideo : guaranteesVideos) {
            guaranteesVideo.setIsDelete(1);
            guaranteesVideoMapper.updateByPrimaryKeySelective(guaranteesVideo);
        }
        GuaranteesVideo guaranteesVideo = new GuaranteesVideo();
        guaranteesVideo.setDate(date);
        guaranteesVideo.setStaffId(staffId);
        guaranteesVideo.setVideoIds(JSONObject.toJSONString(videoIds));
        guaranteesVideoMapper.insertSelective(guaranteesVideo);
    }


    public Map<Long, VideoDetail> getVideoDetail(Set<Long> videoIdList) {
        try {
            Map<Long, VideoDetail> map = new HashMap<>();
            JSONObject params = new JSONObject();
            params.put("videoIdList", videoIdList);
            String post = httpPoolClient.post(POST_VIDEO_DETAIL_URL, params.toJSONString());
            JSONObject res = JSONObject.parseObject(post);
            JSONArray data = res.getJSONArray("data");
            for (int i = 0; i < data.size(); i++) {
                VideoDetail videoDetail = new VideoDetail();
                JSONObject jsonObject = data.getJSONObject(i);
                Long videoId = jsonObject.getLong("id");
                String shareImgPath = jsonObject.getString("shareImgPath");
                String title = jsonObject.getString("title");
                videoDetail.setCover(shareImgPath);
                videoDetail.setTitle(title);
                map.put(videoId, videoDetail);
            }
            return map;
        } catch (IOException e) {
            log.error("MessageAttachmentServiceImpl getVideDetail error", e);
        }
        return new HashMap<>();
    }


}
