package com.tzld.piaoquan.offline.service.Impl;

import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.common.common.constant.MessageConstant;
import com.tzld.piaoquan.common.component.HttpPoolClient;
import com.tzld.piaoquan.offline.service.OfflineAccessTokenService;
import com.tzld.piaoquan.offline.service.OfflineMessageService;
import com.tzld.piaoquan.common.utils.DateUtil;
import com.tzld.piaoquan.common.utils.LarkRobotUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;

import static com.tzld.piaoquan.common.common.constant.WeComConstant.POST_WE_COM_ADD_MSG_TEMPLATE;


@Slf4j
@Service
public class OfflineMessageServiceImpl implements OfflineMessageService {

    @Autowired
    private HttpPoolClient httpPoolClient;

    @Autowired
    private OfflineAccessTokenService offlineAccessTokenService;

    @Override
    public boolean pushWeComMessage(JSONObject jsonObject, Long corpId) {
        try {
            String accessToken = offlineAccessTokenService.getWeComAccessToken(corpId);
            String url = POST_WE_COM_ADD_MSG_TEMPLATE
                    + "?access_token=" + accessToken;
            String s = httpPoolClient.post(url, jsonObject.toJSONString());
            JSONObject res = JSONObject.parseObject(s);
            log.info("pushWeComMessage res={}", res);
            Integer code = res.getInteger("errcode");
            if (code != 0) {
                LarkRobotUtil.sendMessage("pushWeComMessage error" + "res:" + res.toJSONString());
            }
            return code == 0;
        } catch (IOException e) {
            LarkRobotUtil.sendMessage("pushWeComMessage error" + e.getMessage());
            log.error("pushWeComMessage error", e);
        }
        return false;
    }




    @Override
    public String getMessageText() {
        Integer hour = DateUtil.getHourOfDay();
        if (hour < 12) {
            return MessageConstant.morningText;
        } else {
            return MessageConstant.guaranteedText;
        }
    }
}
